

# Compute PAVA algorithm from  landmarks estimations
obtainPava<-function(x,y,iL,iU,timePoints){
  require(Iso)
  nTimePointsPeriod<-timePoints/2 # number of time points per period

  # different options from landmark positions
  # option a
  if( iL < iU & ((iL!=1 & iU==nTimePointsPeriod)|
                   (iL==1 & iU!=nTimePointsPeriod)|(iL!=1 & iU!=nTimePointsPeriod))){


    xLU<-x[iL:iU]
    yLU<-y[iL:iU]
    
    if(iU==nTimePointsPeriod){
      iUsig<-1
    }else{
      iUsig<-iU+1
    }
    if(iL==1){
      iLant<-nTimePointsPeriod
    }else{
      iLant<-iL-1
    }
    if(iUsig>iLant){
      yUL<-y[c(iUsig:nTimePointsPeriod,1:iLant)]
      xUL<-x[c(iUsig:nTimePointsPeriod,1:iLant)]
    }else{
      yUL<-y[c(iUsig:iLant)]
      xUL<-x[c(iUsig:iLant)]
    }  
    yLUstar<-pava(yLU)
    xLUstar<-pava(xLU)
    yULstar<-pava(yUL,decreasing=TRUE)
    xULstar<-pava(xUL,decreasing=TRUE)

    ys<-c(yLU,yUL)
    xs<-c(xLU,xUL)
    ystar<-c(yLUstar,yULstar)
    xstar<-c(xLUstar,xULstar)
    data1In1 <- sort(ystar)
    if (length(data1In1)>1){
      data2In1 <- c(data1In1[-1], data1In1[1])
      differenceIn1 <- ifelse((round(data1In1 - data2In1,13) == 0), 0, 1)
    } else {
      differenceIn1 <- 1
    }
    if(sum(differenceIn1)==0){
      m1<-1
    }else{
      m1 <- sum(differenceIn1)
    }
  } 
  # option b
  if( iU < iL ){
    
    yLU<-y[c(iL:nTimePointsPeriod,1:iU)]
    xLU<-x[c(iL:nTimePointsPeriod,1:iU)]
    if(length(yLU)==nTimePointsPeriod){
      ys<-yLU
      ystar<-pava(yLU)
      xs<-xLU
      xstar<-pava(xLU)
    }else{
      yUL<-y[(iU+1):(iL-1)]
      xUL<-x[(iU+1):(iL-1)]
      yLUstar<-pava(yLU)
      xLUstar<-pava(xLU)
      yULstar<-pava(yUL,decreasing=TRUE)
      xULstar<-pava(xUL,decreasing=TRUE)
      
      ys<-c(yLU,yUL)
      xs<-c(xLU,xUL)
      ystar<-c(yLUstar,yULstar)
      xstar<-c(xLUstar,xULstar)
    }
    
    data1In1 <- sort(ystar)
    if (length(data1In1)>1){
      data2In1 <- c(data1In1[-1], data1In1[1])
      differenceIn1 <- ifelse((round(data1In1 - data2In1,13) == 0), 0, 1)
    } else {
      differenceIn1 <- 1
    }
    if(sum(differenceIn1)==0){
      m1<-1
    }else{
      m1 <- sum(differenceIn1)
    }
  } 
  # option C
  if( iL < iU & iL==1 & iU==nTimePointsPeriod){
    
    yLU<-y[iL:iU]
    xLU<-x[iL:iU]
   
    ys<-yLU
    xs<-xLU
    ystar<-pava(yLU)
    xstar<-pava(xLU)
    data1In1 <- sort(ystar)
    if (length(data1In1)>1){
      data2In1 <- c(data1In1[-1], data1In1[1])
      differenceIn1 <- ifelse((round(data1In1 - data2In1,13) == 0), 0, 1)
    } else {
      differenceIn1 <- 1
    }
    if(sum(differenceIn1)==0){
      m1<-1
    }else{
      m1 <- sum(differenceIn1)
    }    
  }
  if(iL==iU){
    xs<-x
    ys<-y
    xstar<-xs
    ystar<-ys
    m1<-1
  }
  return(list(cbind(xs,xstar,ys,ystar),m1))
}



orderLU<-function(c1,c2,reverse=FALSE){
  
  x<-c1
  y<-c2
  if(reverse==TRUE){
    x<-c2
    y<-c1
  }
  
  n<-length(x)
  
  iL<-which.min(apply(cbind(c1,c2),1,mean))
  iU<-which.max(apply(cbind(c1,c2),1,mean))

  
  mi<-apply(cbind(x,y),1,mean)
  s2M2<-sum((c1-mi)^2+(c2-mi)^2)/(2*n)
  
  
  ys<-obtainPava(x,y,iL,iU,2*n)[[1]][,3]
  ystar<-obtainPava(x,y,iL,iU,2*n)[[1]][,4]
  
  
  sigma2<-s2M2
  
  est01s2<-(1/sigma2)*sum((ystar-mean(ys))^2) 
  
  m<-obtainPava(x,y,iL,iU,2*n)[[2]]
  gl01<-m-1
  if(gl01==0){
    pval01s2<-1
  }else{
    pval01s2<-pchisq(est01s2,gl01,lower.tail=F)*(1-(1/factorial(n)))
  }
  

  est01<-est01s2
  pval01<-pval01s2
 
  return(c(sigma2,gl01,est01,pval01,iL,iU))
}


FilterClass<-function(M,lF,lQC=0.01,lC,adjFdrB=TRUE,adjFdrD=TRUE,indL,indU,T,timepoints,interval,nPeriods){
  
  
  alpha<-lF
  pasaFiltro<-c()
  
  # inizializating counting variables...
  contNPNC<-0
  contPNC<-0
  contPC<-0
  contPF<-0
  contVBumpy<-0


  listaPvalNP<-c()
  listaPvalPF<-c()
  listaPvalVBumpy<-c()
  listaVBumpy<-c()
  listaPF<-c()

  # Filtering step
  
  # FDR adjustment
  pT01<-M[,c(5,8)]
  pmax<-apply(pT01,1,max)
  if(adjFdrB==TRUE){
    pvalAux<-c()
    pmax<-p.adjust(pmax,method="BH")
    for(i in 1:(nrow(M))){
      if(pmax[i]<=lF){
        pasaFiltro[i]<-TRUE
      }else{
        pasaFiltro[i]<-FALSE
        pvalAux[i]<-pmax[i]
        listaPvalNP<-c(listaPvalNP,pvalAux[i])
      }
    }
  }
  
  # non FDR adjustment
  if(adjFdrB==FALSE){
    pvalAux<-pmax
    for(i in 1:nrow(M)){
      
      if(pvalAux[i]<=alpha ){
        pasaFiltro[i]<-TRUE
      }else{
        pasaFiltro[i]<-FALSE
        pvalAux[i]<-max(pT01[i,1],pT01[i,2])
        listaPvalNP<-c(listaPvalNP,pvalAux[i])
      }
    }
  }
  

  for(j in 1:nrow(M)){
    if(pasaFiltro[j]==FALSE){
      contNPNC<-contNPNC+1
    }
  }

  N<-sum(pasaFiltro)

  if(sum(pasaFiltro)!=0){
    
    genesPer<-setdiff(1:nrow(M),which(pasaFiltro==FALSE))
    
    indBumpy<-c()
    for(j in 1:nrow(M)){
      if(pasaFiltro[j]==FALSE ){
        indBumpy<-c(indBumpy,j)
      }
    }
    
  }else{
    indBumpy<-1:nrow(M)
  }
  
  # Re-capturing and adapting the data 
  
  d<-matrix(0,nrow(M),48)
  
  for(j in 1:nrow(M)){
    d1<-lista[[j]][1:24] # related to KEY EXECUTION of main routine
    d2<-lista[[j]][25:48] # related to KEY EXECUTION of main routine
    d[j,]<-c(d1,d2)
  }
  
  d<-as.matrix(d)
  
    
  # Classifyng already non-filtered genes
  listBumpy<-list()
  pvalListBumpy<-list()
  pvalFlatBumpy<-c()
  pvalCyclicBumpy<-c()
  pvalBumpy<-c()
  indFlatBumpy<-c()
  indCyclicBumpy<-c()
  
  
  if(length(indBumpy)>0){
    for(i in 1:length(indBumpy)){
      listBumpy[[i]]<-as.numeric(d[indBumpy[i],])
      pvalListBumpy[[i]]<-ORI(timepoints,nPeriods,unlist(listBumpy[[i]]),indexL=indL[indBumpy[i]],indexU=indU[indBumpy[i]])
      pvalBumpy[i]<-as.numeric(pvalListBumpy[[i]][[2]])  
    }
    for(i in 1:length(pvalBumpy)){
      if(is.na(pvalBumpy[i])){
        pvalBumpy[i]<-1
      }
    }
    pvalBumpy<-p.adjust(pvalBumpy,method="BH")
    for(i in 1:length(indBumpy)){
      if(pvalBumpy[i]<=lC ){
        indCyclicBumpy<-c(indCyclicBumpy,indBumpy[i])
        pvalCyclicBumpy<-c(pvalCyclicBumpy,pvalBumpy[i])
      }
      if(pvalBumpy[i]>lC){
        indFlatBumpy<-c(indFlatBumpy,indBumpy[i])
        pvalFlatBumpy<-c(pvalFlatBumpy,pvalBumpy[i])
      }
    }
    if(length(indCyclicBumpy)==0){
      contVBumpy<-0
      listaVBumpy<-c()
      listaPvalVBumpy<-c()

    }else{
      for(i in 1:length(indCyclicBumpy)){
        linea1<-which(indBumpy==indCyclicBumpy[i])
        listaVBumpy<-c(listaVBumpy,indBumpy[linea1])
        listaPvalVBumpy<-c(listaPvalVBumpy,listaPvalNP[linea1])
        contVBumpy<-contVBumpy+1

      }
    }
    if(length(indFlatBumpy)!=0){
      for(i in 1:length(indFlatBumpy)){
        linea2<-which(indBumpy==indFlatBumpy[i])
        listaPF<-c(listaPF,indBumpy[linea2])
        listaPvalPF<-c(listaPvalPF,listaPvalNP[linea2])
        contPF<-contPF+1
      }
    }
  }else{
    contVBumpy<-0
    listaVBumpy<-c()
    listaPvalVBumpy<-c()
  }
  
  if(sum(pasaFiltro!=0)){ 
    candidatos<-genesPer
    listaNP<-setdiff(1:nrow(M),candidatos)
    listaPNC<-c()
    listaPC<-c()
    listaPvalPNC<-c()
    listaPvalPC<-c()

    
    # Deleting non-filtered genes
    if(length(genesPer)==1){
      d<-matrix(d[genesPer,],1,48)
    }else{
      d<-d[genesPer,]
    }
    
    ejemplo<-list()
    ejemploE<-list()
    
    for(i in 1:nrow(d)){
      ejemplo[[i]]<-as.numeric(d[i,])
      ejemploE[[i]]<-ORI(timepoints,nPeriods,unlist(ejemplo[[i]]),indexL=indL[genesPer[i]],indexU=indU[genesPer[i]])
    }
    
    # FDR adjustment
    if(adjFdrD==TRUE){
      pH1H2<-c()
      pH0H1<-c()
      for(i in 1:nrow(d)){
        pH1H2[i]<-as.numeric(ejemploE[[i]][[1]])
        pH0H1[i]<-as.numeric(ejemploE[[i]][[2]])
      }
      write.table(pH1H2,"ph1h2sinfdr.txt")

      if(sum(pH1H2<=lQC)==0){
        indpH0H1<-genesPer
      }else{
        indpH0H1<-genesPer[-which(pH1H2<=lQC)]
      }

      for(i in 1:length(genesPer)){
        if(pH1H2[i]<=lQC ){
          contPNC<-contPNC+1
          listaPNC<-c(listaPNC,candidatos[i])
          listaPvalPNC<-c(listaPvalPNC,pH1H2[i])
        }
      }
      
      N<-length(indpH0H1)
      if(sum(pH1H2<=lQC)==0){
        pH0H1A<-pH0H1
      }else{
        pH0H1A<-pH0H1[-which(pH1H2<=lQC)]
      }

      candidatos<-indpH0H1
      pH0H1A<-p.adjust(pH0H1A,method="BH")
      for(i in 1:length(indpH0H1)){
        
        if( pH0H1A[i]<=lC){
          contPC<-contPC+1
          listaPC<-c(listaPC,candidatos[i])
          listaPvalPC<-c(listaPvalPC,pH0H1A[i])
          
        }else{
          contPF<-contPF+1
          
          listaPF<-c(listaPF,candidatos[i])
          listaPvalPF<-c(listaPvalPF,pH0H1A[i])
        }
        
        
      }
    }
    
    
    # non FDR adjustment
    if(adjFdrD==FALSE){
      pH1H2<-c()
      pH0H1<-c()
      for(i in 1:nrow(d)){
        pH1H2[i]<-as.numeric(ejemploE[[i]][[2]][,,1])
        pH0H1[i]<-as.numeric(ejemploE[[i]][[3]][,,1])
      }
      if(sum(pH1H2<=lQC)==0){
        indpH0H1<-genesPer
      }else{
        indpH0H1<-genesPer[-which(pH1H2<=lQC)]
      }
      
      
      for(i in 1:length(genesPer)){
        if(pH1H2[i]<=lQC ){
          contPNC<-contPNC+1
          listaPNC<-c(listaPNC,candidatos[i])
          listaPvalPNC<-c(listaPvalPNC,pH1H2[i])
        }
      }
      
      candidatos<-indpH0H1
      for(i in 1:length(indpH0H1)){
        if(pH0H1[i]<=alpha ){
          contPC<-contPC+1
          listaPC<-c(listaPC,candidatos[i])
          listaPvalPC<-c(listaPvalPC,pH0H1[i])
        }
        if( pH0H1[i]>alpha ){
          contPF<-contPF+1
          listaPF<-c(listaPF,candidatos[i],pH0H1[i])
          listaPvalPF<-c(listaPvalPF,pH0H1[i])
        }
      }
    }
  }
  R<-rbind(contPC,contPNC,contVBumpy,contPF)
  rownames(R)<-c("CYC","QCYC","NFNP","FLAT")

  
	
  # Saving outputs in text files
  write.table(listaVBumpy,   "listNFNP.txt",sep="\t",dec=",")
  write.table(listaPC,   "listC.txt",sep="\t",dec=",")
  write.table(listaPNC,  "listQC.txt",sep="\t",dec=",")
  write.table(listaPF,   "listF.txt",sep="\t",dec=",")
  write.table(listaPvalVBumpy,   "listPvalNFNP.txt",sep="\t",dec=",")
  write.table(listaPvalPC,   "listPvalC.txt",sep="\t",dec=",")
  write.table(listaPvalPNC,  "listPvalQC.txt",sep="\t",dec=",")
  write.table(listaPvalPF,   "listPvalF.txt",sep="\t",dec=",")


  return(R)
}


# Conduct  testing problems
ORI<-function(timepoints=timepoints,nPeriods=nPeriods,dat=dat,indexL=indexL,indexU=indexU){
  
  # preparing data
  data <- matrix(dat,timepoints/2,nPeriods)
  l<-indexL
  u<-indexU

  d1<-((u-l)%%(timepoints/2))+1
  d2<-(timepoints/2)-d1
        
  yLU<- c(0)
  obsLU<-matrix(0,d1,2)
        
        if(d2!=0){
          obsUL<-matrix(0,d2,2)
          yUL<-c(0)
        }
        
        datEj <- data
        datEjm <- matrix(apply(datEj,1,mean),(timepoints/2),1)
        
        if(d2!=0){
          y2<-c(datEjm,datEjm)
          obs2<-rbind(datEj,datEj)
          for(i in 1:d1){
            obsLU[i,]<-obs2[l+i-1,]
            yLU[i]<-y2[l+i-1]
          }
          for(i in 1:d2){
            obsUL[i,]<-obs2[u+i,]
            yUL[i]<-y2[u+i]
          }

          if(length(c(yLU,yUL))!=(timepoints/2)){
            print("error")
          }
          p12E<-h1(obsLU,obsUL,2,yLU,yUL,pava(yLU),pava(yUL,decreasing=T))[1]
          p01E<-h0(obsLU,obsUL,2,yLU,yUL,pava(yLU),pava(yUL,decreasing=T))[1]
        }else{
          y2<-c(datEjm,datEjm)
          obs2<-rbind(datEj,datEj)
          for(i in 1:d1){
            obsLU[i,]<-obs2[l+i-1,]
            yLU[i]<-y2[l+i-1]
          }
          if(length(yLU)!=(timepoints/2)){
            print("error")
          }
          p12E<-h1bis(as.matrix(obsLU),2,yLU,pava(yLU))[1]
          p01E<-h0bis(as.matrix(obsLU),2,yLU,pava(yLU))[1]
                    
        }

  return(list(p12E,p01E))
  
}

# Supporting routines for ORI routine
h0<-function(datosLU,datosUL,omega,muLU,muUL,starLU,starUL){

  mu<-mean(c(muLU,muUL))
  mu1<-mean(c(muLU))
  mu2<-mean(c(muUL))
  
  num<-sum(omega*(starLU-mu)^2)+sum(omega*(starUL-mu)^2)

  den<-sum((datosLU[,1]-mu)^2)+sum((datosLU[,2]-mu)^2)+sum((datosUL[,1]-mu)^2)+sum((datosUL[,2]-mu)^2)

  Th01<-num/(den)
    aux1<-sort(c(starLU,starUL))
  aux2<-c(aux1[-1],aux1[1])
  difference<-ifelse((aux1-aux2)==0,0,1)
  if(sum(difference)==0){
    m<-1
  } else {
    m<-sum(difference)
  }
  gl1<-(m-1)/2
  gl2<-(2*length(c(muLU,muUL))-m)/2
  nn<-length(c(muLU,muUL))-2
  
  p<-pbeta(Th01,gl1,gl2,lower.tail=F)*(1-(1/factorial(nn)))
  return(c(p))
}

h0bis<-function(datosLU,omega,muLU,starLU){
   
  mu<-mean(muLU)
  mu1<-mu
  mu2<-NA
  num<-sum(omega*(starLU-mu)^2)
  den<-sum((datosLU[,1]-mu)^2)+sum((datosLU[,2]-mu)^2)
  
  Th01<-num/den
  aux1LU<-sort(starLU)
  aux2LU<-c(aux1LU[-1],aux1LU[1])
  differenceLU<-ifelse((aux1LU-aux2LU)==0,0,1)
 
  if(sum(differenceLU)==0){
    mLU<-1
  } else {
    mLU<-sum(differenceLU)
  }
  m<-mLU
  gl1<-(m-1)/2
  gl2<- (2*length(muLU)-m)/2

  nn<-length(c(muLU))-2
  p<-pbeta(Th01,gl1,gl2,lower.tail=F)*(1-(1/factorial(nn)))

  return(c(p))
}


h1<-function(datosLU,datosUL,omega,muLU,muUL,starLU,starUL){
  
  
  num<-sum(omega*(muLU-starLU)^2)+sum(omega*(muUL-starUL)^2)

  d1<-c(datosLU[,1],datosUL[,1])
  d2<-c(datosLU[,2],datosUL[,2])
  s<-c(starLU,starUL)
  den<-sum((d1-s)^2)+sum((d2-s)^2)
  T<-num/den
  
  aux1<-sort(c(starLU,starUL))
  aux2<-c(aux1[-1],aux1[1])
  difference<-ifelse((aux1-aux2)==0,0,1)
  if(sum(difference)==0){
    m<-1
  } else {
    m<-sum(difference)
  }
  
  

  gl1<-(length(c(muLU,muUL))-m)/2
  gl2<-(length(c(muLU,muUL)))/2
  nn<-length(c(muLU,muUL))-2
  p<-pbeta(T,gl1,gl2,lower.tail=F)*(1-(1/factorial(nn)))
  return(c(p))
}

h1bis<-function(datosLU,omega,muLU,starLU){
  
  num<-sum(omega*(muLU-starLU)^2)
  d1<-c(datosLU[,1])
  d2<-c(datosLU[,2])
  s<-c(starLU)
  den<-sum((d1-s)^2)+sum((d2-s)^2)
  T<-num/den
  
  aux1LU<-sort(starLU)
  aux2LU<-c(aux1LU[-1],aux1LU[1])
  differenceLU<-ifelse((aux1LU-aux2LU)==0,0,1)
  if(sum(differenceLU)==0){
    mLU<-1
  } else {
    mLU<-sum(differenceLU)
  }
  
  
  
  m<-mLU
  gl1<-(length(muLU)-m)/2
  gl2<-(length(muLU))/2
  nn<-length(c(muLU))-2
  p<-pbeta(T,gl1,gl2,lower.tail=F)*(1-1/(factorial(nn)))

  return(c(p))
}





